import {Component, OnInit} from '@angular/core'
import {FormBuilder, FormControl, FormGroup, Validators} from '@angular/forms'
import {AccountService} from '../../services/account.service'
import {ActivatedRoute, Router} from '@angular/router'
import {ErrorService} from '../../../shared/service/error.service'
import {ErrorHandlingComponent} from '../../../shared/components/subscribing.component'
import {AppState} from '../../../shared/app-state'

@Component({
  selector: 'cre-login',
  templateUrl: './login.component.html',
  styleUrls: ['./login.component.sass']
})
export class LoginComponent extends ErrorHandlingComponent implements OnInit {
  form: FormGroup
  idFormControl: FormControl
  passwordFormControl: FormControl

  constructor(
    private formBuilder: FormBuilder,
    private accountService: AccountService,
    private appState: AppState,
    errorService: ErrorService,
    router: Router,
    activatedRoute: ActivatedRoute
  ) {
    super(errorService, activatedRoute, router)
    this.appState.title = 'Connexion'
  }

  ngOnInit(): void {
    this.errorService.activeErrorHandler = this

    if (this.accountService.isLoggedIn()) {
      this.router.navigate(['/color'])
    }

    this.idFormControl = this.formBuilder.control(null, Validators.compose([Validators.required, Validators.pattern(new RegExp('^[0-9]+$'))]))
    this.passwordFormControl = this.formBuilder.control(null, Validators.required)
    this.form = this.formBuilder.group({
      id: this.idFormControl,
      password: this.passwordFormControl
    })
  }

  submit() {
    this.accountService.login(
      this.idFormControl.value,
      this.passwordFormControl.value,
      () => this.router.navigate(['/color'])
    )
  }
}
