import {Component, Input} from '@angular/core'
import {Recipe} from '../../../shared/model/recipe.model'
import {SubscribingComponent} from '../../../shared/components/subscribing.component'
import {ActivatedRoute, Router} from '@angular/router'
import {RecipeImageService} from '../../services/recipe-image.service'
import {ErrorService} from '../../../shared/service/error.service'
import {openJpg} from '../../../shared/utils/utils'

@Component({
  selector: 'cre-images-editor',
  templateUrl: './images-editor.component.html',
  styleUrls: ['./images-editor.component.sass']
})
export class ImagesEditorComponent extends SubscribingComponent {
  @Input() recipe: Recipe
  @Input() editionMode = false

  imagesUrls: string[]

  constructor(
    private recipeImageService: RecipeImageService,
    errorService: ErrorService,
    router: Router,
    activatedRoute: ActivatedRoute
  ) {
    super(errorService, activatedRoute, router)
  }

  ngOnInit() {
    super.ngOnInit()

    this.imagesUrls = this.recipe.imagesUrls
  }

  submit(event) {
    const image = event.target.files[0]
    this.subscribe(
      this.recipeImageService.save(image, this.recipe.id),
      r => this.imagesUrls = r.imagesUrls
    )
  }

  openImage(url: string) {
    openJpg(url)
  }

  delete(url: string) {
    this.subscribe(
      this.recipeImageService.delete(url, this.recipe.id),
      () => this.removeUrl(url)
    )
  }

  private removeUrl(url: string) {
    this.imagesUrls = this.imagesUrls.filter(u => u !== url)
  }
}
