import {Component, Input} from '@angular/core'
import {Recipe, RecipeStep, recipeStepsForGroupId, sortRecipeSteps} from '../../../shared/model/recipe.model'
import {MatTable} from '@angular/material/table'
import {Observable} from 'rxjs'
import {Group} from '../../../shared/model/user'

@Component({
  selector: 'cre-step-table',
  templateUrl: './step-table.component.html',
  styleUrls: ['./step-table.component.sass']
})
export class StepTableComponent {
  readonly columns = ['position', 'buttonsPosition', 'message', 'buttonRemove']

  @Input() recipe: Recipe
  @Input() groups$: Observable<Group[]>
  @Input() selectedGroupId: number | null

  hoveredStep: RecipeStep | null

  private groupSteps = new Map<number, RecipeStep[]>()

  addStep(table: MatTable<any>) {
    const addedStep = new RecipeStep(null, '', this.selectedGroupSteps.length + 1)
    this.selectedGroupSteps.push(addedStep)
    table.renderRows()
  }

  removeStep(position: number, table: MatTable<any>) {
    this.selectedGroupSteps.splice(position, 1)

    // Decreases the position of each step above the removed one
    for (let i = position; i < this.selectedGroupSteps.length; i++) {
      this.selectedGroupSteps[i].position -= 1
    }

    table.renderRows()
  }

  increasePosition(step: RecipeStep, table: MatTable<any>) {
    this.updateStepPosition(step, step.position + 1)
    this.sort(table)
  }

  decreasePosition(step: RecipeStep, table: MatTable<any>) {
    this.updateStepPosition(step, step.position - 1)
    this.sort(table)
  }

  sort(table: MatTable<any>) {
    this.groupSteps.set(this.selectedGroupId, sortRecipeSteps(this.selectedGroupSteps))
    table.renderRows()
  }

  selectedGroupStepAtPosition(position: number): RecipeStep {
    return this.selectedGroupSteps.find(s => s.position === position)
  }

  selectedGroupHasStepAtPosition(position: number): boolean {
    return this.selectedGroupStepAtPosition(position) != undefined
  }

  get selectedGroupSteps(): RecipeStep[] {
    if (!this.groupSteps.has(this.selectedGroupId)) {
      this.groupSteps.set(this.selectedGroupId, recipeStepsForGroupId(this.recipe, this.selectedGroupId))
    }
    return this.groupSteps.get(this.selectedGroupId)
  }

  get selectedGroupStepsCount(): number {
    return this.selectedGroupSteps.length
  }

  get mappedUpdatedSteps(): Map<number, RecipeStep[]> {
    const updatedStepsMap = new Map<number, RecipeStep[]>()
    this.recipe.groupsInformation.forEach(i => {
      updatedStepsMap.set(i.group.id, i.steps)
    })

    // Add steps for groups that were not already in the recipe
    this.groupSteps.forEach((steps, groupId) => {
      if (!updatedStepsMap.has(groupId)) {
        updatedStepsMap.set(groupId, steps)
      }
    })
    return updatedStepsMap
  }

  private updateStepPosition(step: RecipeStep, updatedPosition: number) {
    if (!this.selectedGroupHasStepAtPosition(updatedPosition)) {
      step.position = updatedPosition
    } else {
      const conflictingStep = this.selectedGroupStepAtPosition(updatedPosition)
      conflictingStep.position = step.position
      step.position = updatedPosition
    }
  }
}
