import {Injectable} from '@angular/core'
import {ApiService} from '../../shared/service/api.service'
import {convertMixMaterialQuantity, UNIT_MILLILITER} from '../../shared/units'
import {Observable} from 'rxjs'
import {Mix, MixMaterialDto} from '../../shared/model/recipe.model'

@Injectable({
  providedIn: 'root'
})
export class MixService {
  constructor(
    private api: ApiService
  ) {
  }

  get all(): Observable<Mix[]> {
    return this.api.get<Mix[]>('/recipe/mix')
  }

  getById(id: number): Observable<Mix> {
    return this.api.get<Mix>(`/recipe/mix/${id}`)
  }

  saveWithUnits(name: string, recipeId: number, materialTypeId: number, mixMaterials: MixMaterialDto[], units: string): Observable<void> {
    return this.save(name, recipeId, materialTypeId, this.convertMixMaterialsToMl(mixMaterials, units))
  }

  save(name: string, recipeId: number, materialTypeId: number, mixMaterials: MixMaterialDto[]): Observable<void> {
    const body = {
      name,
      recipeId,
      materialTypeId,
      mixMaterials: []
    }
    this.appendMixMaterialsToBody(mixMaterials, body)
    return this.api.post('/recipe/mix', body)
  }

  updateWithUnits(id: number, name: string, materialTypeId: number, mixMaterials: MixMaterialDto[], units: string): Observable<void> {
    return this.update(id, name, materialTypeId, this.convertMixMaterialsToMl(mixMaterials, units))
  }

  update(id: number, name: string, materialTypeId: number, mixMaterials: MixMaterialDto[]): Observable<void> {
    const body = {
      id,
      name,
      materialTypeId,
      mixMaterials: []
    }

    this.appendMixMaterialsToBody(mixMaterials, body)
    return this.api.put('/recipe/mix', body)
  }

  delete(id: number): Observable<void> {
    return this.api.delete(`/recipe/mix/${id}`)
  }

  private convertMixMaterialsToMl(mixMaterials: MixMaterialDto[], units: string): MixMaterialDto[] {
    return mixMaterials.map(m => {
      m.quantity = convertMixMaterialQuantity(m, units, UNIT_MILLILITER)
      return m
    })
  }

  private appendMixMaterialsToBody(mixMaterials: MixMaterialDto[], body: any) {
    mixMaterials.filter(m => m.materialId != null && m.quantity != null).forEach(m => {
      body.mixMaterials.push({
        materialId: m.materialId,
        quantity: m.quantity,
        position: m.position
      })
    })
  }
}

