import {Component} from '@angular/core'
import {MaterialTypeService} from '../../service/material-type.service'
import {ErrorHandlingComponent} from '../../../shared/components/subscribing.component'
import {Permission} from '../../../shared/model/user'
import {ActivatedRoute, Router} from '@angular/router'
import {ErrorService} from '../../../shared/service/error.service'
import {AppState} from '../../../shared/app-state'

@Component({
  selector: 'cre-list',
  templateUrl: './list.component.html',
  styleUrls: ['./list.component.sass']
})
export class ListComponent extends ErrorHandlingComponent {
  materialTypes$ = this.materialTypeService.all
  columns = [
    {def: 'name', title: 'Nom', valueFn: t => t.name},
    {def: 'prefix', title: 'Préfixe', valueFn: t => t.prefix},
    {def: 'usePercentages', title: 'Utilise les pourcentages', valueFn: t => t.usePercentages ? 'Oui' : 'Non'}
  ]
  buttons = [
    {
      text: 'Modifier',
      linkFn: t => `/catalog/materialtype/edit/${t.id}`,
      permission: Permission.EDIT_MATERIAL_TYPES,
      disabledFn: t => t.systemType
    }
  ]

  constructor(
    private materialTypeService: MaterialTypeService,
    private appState: AppState,
    errorService: ErrorService,
    router: Router,
    activatedRoute: ActivatedRoute
  ) {
    super(errorService, activatedRoute, router)
    this.appState.title = 'Types de produit'
  }
}
