import {Component, EventEmitter, Input, Output} from '@angular/core'
import {FormBuilder, FormControl, FormGroup, ValidatorFn, Validators} from '@angular/forms'
import {Observable} from 'rxjs'

@Component({
  selector: 'cre-entity-add',
  templateUrl: './entity-add.component.html',
  styleUrls: ['./entity-add.component.sass']
})
export class EntityAddComponent {
  @Input() title: string
  @Input() backButtonLink: string
  @Input() formFields: FormField[]
  @Output() submit = new EventEmitter<any>()

  form: FormGroup | null

  constructor(
    private formBuilder: FormBuilder
  ) {
  }

  ngOnInit() {
    const formGroup = {}
    this.formFields.forEach(f => {
      const validator = f.required ? Validators.compose([Validators.required, f.validator]) : f.validator
      formGroup[f.name] = new FormControl(f.defaultValue, validator)
    })
    this.form = this.formBuilder.group(formGroup)
  }

  submitForm() {
    const values = {}
    this.formFields.forEach(f => {
      values[f.name] = this.getControl(f.name).value
    })
    this.submit.emit(values)
  }

  getControl(controlName: string): FormControl {
    return this.form.controls[controlName] as FormControl
  }
}

export class FormField {
  constructor(
    public name: string,
    public label?: string,
    public icon?: string,
    public type?: string,
    public required?: boolean,
    public validator?: ValidatorFn,
    public errorMessages?: FormErrorMessage[],
    public valueFn?: (any) => any,
    public template?: any,
    public readonly?: boolean,
    public defaultValue?: any,
    // Specifics to some types
    public step?: string,
    public options$?: Observable<{ value: any, label: string }[]>,
    public fileType?: string,
    public min?: number,
    public max?: number
  ) {
  }
}

export class FormErrorMessage {
  constructor(
    public conditionFn: (any) => boolean,
    public message: string
  ) {
  }
}
