import {
  AfterContentInit,
  Component,
  ContentChildren,
  Directive,
  HostBinding,
  Input,
  QueryList,
  ViewChild,
  ViewEncapsulation
} from '@angular/core'
import {MatColumnDef, MatHeaderRowDef, MatRowDef, MatTable} from '@angular/material/table'

@Directive({
  selector: '[creInteractiveCell]'
})
export class CreInteractiveCell implements AfterContentInit {
  @Input('creInteractiveCell') index: number

  @HostBinding() hidden = true

  private _selectedIndex = 0
  private _hoverIndex = 0

  ngAfterContentInit(): void {
    this.hidden = this.isHidden
  }

  set hoverIndex(index: number) {
    this._hoverIndex = index
    this.hidden = this.isHidden
  }

  set selectedIndex(index: number) {
    this._selectedIndex = index
    this.hidden = this.isHidden
  }

  get isHidden(): boolean {
    return this._hoverIndex !== this.index && this._selectedIndex !== this.index
  }
}

@Component({
  selector: 'cre-table',
  templateUrl: 'table.html',
  styleUrls: ['table.sass'],
  encapsulation: ViewEncapsulation.None
})
export class CreTable<T> implements AfterContentInit {
  @ContentChildren(MatHeaderRowDef) headerRowDefs: QueryList<MatHeaderRowDef>
  @ContentChildren(MatRowDef) rowDefs: QueryList<MatRowDef<T>>
  @ContentChildren(MatColumnDef) columnDefs: QueryList<MatColumnDef>

  @ContentChildren(CreInteractiveCell, {descendants: true}) interactiveCells: QueryList<CreInteractiveCell>

  @ViewChild(MatTable, {static: true}) table: MatTable<T>

  @Input() columns: string[]
  @Input() dataSource: T[]
  @Input() interactive = true

  selectedIndex = 0

  ngAfterContentInit(): void {
    this.columnDefs.forEach(columnDef => this.table.addColumnDef(columnDef))
    this.rowDefs.forEach(rowDef => this.table.addRowDef(rowDef))
    this.headerRowDefs.forEach(headerRowDef => this.table.addHeaderRowDef(headerRowDef))
  }

  onRowHover(index: number) {
    if (this.interactive) {
      this.interactiveCells.forEach(cell => cell.hoverIndex = index)
    }
  }

  onRowSelected(index: number) {
    if (this.interactive) {
      this.selectedIndex = index
      this.interactiveCells.forEach(cell => cell.selectedIndex = index)
    }
  }
}
