import {Component} from '@angular/core'
import {Observable} from 'rxjs'
import {UserService} from '../../services/user.service'
import {User, Permission} from '../../../shared/model/user'
import {takeUntil} from 'rxjs/operators'
import {AccountService} from '../../../accounts/services/account.service'
import {animate, state, style, transition, trigger} from '@angular/animations'
import {ErrorHandlingComponent} from '../../../shared/components/subscribing.component'
import {ActivatedRoute, Router} from '@angular/router'
import {ErrorService} from '../../../shared/service/error.service'
import {AppState} from '../../../shared/app-state'

@Component({
  selector: 'cre-users',
  templateUrl: './list.component.html',
  styleUrls: ['./list.component.sass'],
  animations: [
    trigger('detailExpand', [
      state('collapsed', style({height: '0px', minHeight: '0'})),
      state('expanded', style({height: '*'})),
      transition('expanded <=> collapsed', animate('225ms cubic-bezier(0.4, 0.0, 0.2, 1)'))
    ])
  ]
})
export class ListComponent extends ErrorHandlingComponent {
  users$: Observable<User[]>
  columns = [
    {def: 'id', title: 'Numéro d\'utilisateur', valueFn: e => e.id},
    {def: 'name', title: 'Nom', valueFn: e => `${e.firstName} ${e.lastName}`},
    {def: 'group', title: 'Groupe', valueFn: e => e.group ? e.group.name : 'Aucun'},
    {def: 'permissionCount', title: 'Nombre de permissions', valueFn: e => e.permissions.length},
    {def: 'lastLogin', title: 'Dernière connexion', valueFn: e => e.lastLoginTime ? this.getDate(e.lastLoginTime).toLocaleString() : 'Jamais'}
  ]
  buttons = [{
    text: 'Modifier',
    linkFn: user => `/admin/user/edit/${user.id}`,
    permission: Permission.EDIT_USERS
  }, {
    text: 'Modifier mot de passe',
    linkFn: user => `/admin/user/password/edit/${user.id}`,
    permission: Permission.EDIT_USERS
  }]

  constructor(
    private userService: UserService,
    private accountService: AccountService,
    private appState: AppState,
    errorService: ErrorService,
    router: Router,
    activatedRoute: ActivatedRoute
  ) {
    super(errorService, activatedRoute, router)
    this.appState.title = 'Utilisateurs'
  }

  ngOnInit(): void {
    this.users$ = this.userService.all.pipe(takeUntil(this.destroy$))
  }

  getDate(dateString: string) {
    return new Date(dateString)
  }
}
